import random

from controllers.ControllerBullet import ControllerBullet
from controllers.ControllerTank import ControllerTank
from models.Game import Game
from models.GameObject import GameObject
from models.enums.EnumGameObjectType import EnumGameObjectType


class ControllerGame:

    _instance = None # static variable shared by all ControllerGame instances

    def __init__(self):
        if ControllerGame._instance is not None:
            raise Exception("Cannot initialize singleton twice")
        self._game: Game = None # non-static member variable

    @staticmethod
    def instance():
        if ControllerGame._instance is None:
            ControllerGame._instance = ControllerGame()
        return ControllerGame._instance

    @property
    def game(self):
        return self._game

    def set_game(self, game):
        self._game = game

    def new_game(self):
        self._game = game = Game()

        for x in range(game.map_size[0]):
            for y in range(game.map_size[1]):

                # randomly choose map tile type
                game_object_options = [EnumGameObjectType.NotSet] * 50 + [
                    EnumGameObjectType.Forest,
                    EnumGameObjectType.Water,
                    EnumGameObjectType.Brick,
                    EnumGameObjectType.Steel
                ] * 2 + [EnumGameObjectType.Enemy]
                game_object_type = random.choice(game_object_options)

                if x == int(game.map_size[0] / 2) and y == game.map_size[1] - 1:
                    game_object_type = EnumGameObjectType.Player

                if game_object_type != EnumGameObjectType.NotSet:
                    game_object = GameObject(
                        position=[x, y],
                        game_object_type=game_object_type,
                    )
                    game.game_objects.append(game_object)

                    game_object.animation_frame_duration = random.randint(0, int(game_object.animation_frame_duration_max))
                    game_object.animation_frame_max = 1
                    if game_object_type in [EnumGameObjectType.Water, EnumGameObjectType.Enemy, EnumGameObjectType.Player]:
                        game_object.animation_frame_max = 2
                        game_object.animation_frame_duration_max = 500
                        if game_object_type not in [EnumGameObjectType.Enemy, EnumGameObjectType.Player]:
                            game_object.animation_is_animating = True
                        else:
                            game_object.animation_frame_duration_max = 200


    def update(self, delta_time: float):
        game = self._game
        for game_object in game.game_objects:
            if game_object.game_object_type in [EnumGameObjectType.Enemy, EnumGameObjectType.Player]:
                ControllerTank.update(game_object, game, delta_time)
            elif game_object.game_object_type == EnumGameObjectType.Bullet:
                ControllerBullet.update(game_object, game, delta_time)

            if game_object.animation_is_animating:
                game_object.animation_frame_duration += delta_time
                if game_object.animation_frame_duration >= game_object.animation_frame_duration_max:
                    game_object.animation_frame_duration = 0
                    game_object.animation_frame += 1
                    if game_object.animation_frame >= game_object.animation_frame_max:
                        game_object.animation_frame = 0
